#!/usr/bin/env python
# coding: utf-8

###############################################################################################################################

import sys
import root_pandas
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import glob
import math
import ROOT
ROOT.gROOT.SetBatch(True)
ROOT.RooRandom.randomGenerator().SetSeed(42)

###############################################################################################################################

def getSignalParameters(mass = None,
                       righttail = None,
                       lefttail = None,
                       whatkindofmass = 'Pippo'):
    masstouse_as_int = -42
    if whatkindofmass!='Diphoton' and whatkindofmass!='Recoil':
        print('WRONG INPUT FOR THE whatkindofmass PARAMETER!')
        sys.exit(0)
    if whatkindofmass=='Diphoton':
        masstouse_as_int=1
    elif whatkindofmass=='Recoil':
        masstouse_as_int=2
    
    if whatkindofmass=='Diphoton':
        dfpar = df_signal_cb_gg
        dfpar_smeared = df_signal_cb_smeared_gg
    elif whatkindofmass=='Recoil':
        dfpar = df_signal_cb_recoil
        dfpar_smeared = df_signal_cb_smeared_recoil
    else:
        print("Something is VERY wrong... (2)")
        sys.exit(0)
        
    listOfTruth = []
    for X in range(len(dfpar['ALPmass'])):
        listOfTruth.append(math.isclose(dfpar['ALPmass'][X], mass, rel_tol=1e-6))
    listOfTruth_smeared = []
    for X in range(len(dfpar_smeared['ALPmass'])):
        listOfTruth_smeared.append(math.isclose(dfpar_smeared['ALPmass'][X], mass, rel_tol=1e-6))
    
    mean = dfpar.loc[listOfTruth].Mu.values[0] 
    sigma = dfpar_smeared.loc[listOfTruth_smeared].Sigma_Sme_Fit.values[0]
    sigmaSyst = dfpar_smeared.loc[listOfTruth_smeared].Sigma_SemiAbsDiff_Fit.values[0]
    alpha = dfpar.loc[listOfTruth].Alpha.values[0]
    n = dfpar.loc[listOfTruth].N.values[0]
    
    minmass = mean - sigma * lefttail
    maxmass = mean + sigma * righttail
        
    if whatkindofmass=='Diphoton':
        if maxmass > MaxAllowedMassDiphoton:
            maxmass = MaxAllowedMassDiphoton
        if minmass < MinAllowedMassDiphoton:
            minmass = MinAllowedMassDiphoton
    if whatkindofmass=='Recoil':
        if maxmass > MaxAllowedMassRecoil:
            maxmass = MaxAllowedMassRecoil
        if minmass < MinAllowedMassRecoil:
            minmass = MinAllowedMassRecoil
        
    masswindow = maxmass - minmass
        
    d = {
         "mean":mean,
         "alpha":alpha,
         "n":n,
         "sigma":sigma,
         "sigmaSyst":sigmaSyst,
         "minmass":minmass,
         "maxmass":maxmass,
         "masswindow":masswindow,
         "masstouse_as_int":masstouse_as_int,
        }
    
    return d

###############################################################################################################################
###############################################################################################################################

kindofmass = 'Recoil'
righttailwindow = 25
lefttailwindow = 25

MinAllowedMassDiphoton = 0.0
MaxAllowedMassDiphoton = 104
MinAllowedMassRecoil = -3
MaxAllowedMassRecoil = 101.5

thresholdDegreeDiphoton = 0.5
thresholdDegreeRecoil = 8.9
degreeLowDiphoton = 2
degreeHighDiphoton = 3
degreeLowRecoil = 4
degreeHighRecoil = 4

bkg_degree = -1
cs_GGBkg =    4.0165 #nb
nbins = 100
Luminosity = 496731 #nb-1
singlePhotonEfficiency = 0.97
ShowThePlotsInsteadOfPlottingThem = 0

minRecoilMass = 5.0
maxDiphotonMass = 8.0
xsec_min = 0
xsec_max = 6
initialCrossSection = 1
NPointsToScan = 200
ParamOfInterestMin = 0
ParamOfInterestMax = 4

SavingPath = './'

list_of_mass_sig = [
    8.6, 8.642825, 8.68544, 8.7, 8.742491, 8.784777, 8.8, 8.839308, 8.878443, 8.9, 8.938228, 8.976293, 9.0, 9.035641, 9.071143, 9.1, 9.132791, 9.165464, 9.198021, 9.2, 9.23115, 9.262194, 9.293135, 9.3, 9.330272, 9.360445, 9.390522, 9.4, 9.428729, 9.457371, 9.485926, 9.5, 9.527419, 9.55476, 9.582022, 9.6, 9.625252, 9.650439, 9.67556, 9.7, 9.722684, 9.745315, 9.767894, 9.790421, 9.8]

list_of_mass_sig = [X for X in list_of_mass_sig if X >= minRecoilMass]

###############################################################################################################################

# Signal information
df_signal_cb_gg = pd.read_csv('./InputsForExtraction/CBParams_DiphotonSq.txt', sep=' ', skipinitialspace=True)
df_signal_cb_recoil = pd.read_csv('./InputsForExtraction/CBParams_RecoilSq.txt', sep=' ', skipinitialspace=True)
df_signal_cb_smeared_gg = pd.read_csv('./InputsForExtraction/Systematics_DefaultVsSmeared_DiphotonSq_fit.txt', sep='\t', skipinitialspace=True)
df_signal_cb_smeared_recoil = pd.read_csv('./InputsForExtraction/Systematics_DefaultVsSmeared_RecoilSq_fit.txt', sep='\t', skipinitialspace=True)

###############################################################################################################################

UpperLimits_Computed_List = []
UpperLimits_List = []
UpperLimits_EffForThiMass_List = []
UpperLimits_Minu1Sigma_List = []
UpperLimits_Plus1Sigma_List = []
UpperLimits_Minu2Sigma_List = []
UpperLimits_Plus2Sigma_List = []
NLL_Obs_List = []
NLL_0Sig_List = []
PValueMaybe_List = []
PValueMaybeInSigmas_List = []
SignificanceSelfDefined_List = []
Status_List = []

###############################################################################################################################
###############################################################################################################################

for MMM in range(len(list_of_mass_sig)):
    mass_sig = list_of_mass_sig[MMM]
    print('ANALYZED MASS =', mass_sig)
    
    bkg_degree = -1
    if kindofmass=='Diphoton':
        if mass_sig < thresholdDegreeDiphoton:
            bkg_degree = degreeLowDiphoton
        else:
            bkg_degree = degreeHighDiphoton
    elif kindofmass=='Recoil':
        if mass_sig < thresholdDegreeRecoil:
            bkg_degree = degreeLowRecoil
        else:
            bkg_degree = degreeHighRecoil

    ###############################################
    
    # ALP mass
    d = getSignalParameters(mass_sig, whatkindofmass=kindofmass, righttail=righttailwindow, lefttail=lefttailwindow)
    myrange=(d["minmass"], d["maxmass"])

    ###############################################################################################################################

    # Just to get the proper string
    mass_string_to_extract_dfs = 'Forty-two'
    dic_masstouse_int = d["masstouse_as_int"]
    if (kindofmass=='Diphoton' and dic_masstouse_int==1):
        mass_string_to_extract_dfs = 'ALP_InvM_sq'
    elif (kindofmass=='Recoil' and dic_masstouse_int==2):
        mass_string_to_extract_dfs = 'ALP_RecoilM_Ecms_sq'
    else:
        print("Something is VERY wrong")
        print('kindofmass =', kindofmass)
        print('dic_masstouse_int =', dic_masstouse_int)
        sys.exit(0)

    #----------------------------------------------------------------------

    # RooFit workspace
    w = ROOT.RooWorkspace("w")
    VarToUseAndPlot = w.factory(mass_string_to_extract_dfs+'['+str(myrange[0])+','+str(myrange[1])+']')
    w.Print()

    #----------------------------------------------------------------------
    
    title = "m^{2}(diphoton)" if mass_string_to_extract_dfs == 'ALP_InvM_sq' else "m^{2}(recoil)"
    x = ROOT.RooRealVar(mass_string_to_extract_dfs, title, myrange[0], myrange[1], 'GeV^{2}/c^{4}')
    x.setBins(nbins)

    fileToBeRead = ROOT.TFile("./InputsForExtraction/MinimalGG.root")
    wood = fileToBeRead.Get('my_ttree')
    roo_bg = ROOT.RooDataSet("roo_bg", "roo_bg", wood, ROOT.RooArgSet(x))

    dh_bg = ROOT.RooDataHist("dh_bg", "dh_bg", ROOT.RooArgSet(x), roo_bg)
    
    nbg = dh_bg.sum(ROOT.kFALSE)
    
    ###########################################################################################################################

    # catch signal efficiency
    txtWithEffs = pd.read_csv('./InputsForExtraction/SignalEfficiencies.txt', delim_whitespace=True, header=None, skiprows=[0])
    txtWithEffs.columns = ["ALPmass", "Eff_Dip", "Eff_Rec", "minmass_Dip", "maxmass_Dip", "minmass_Rec", "maxmass_Rec"]
    listOfMass_txt = txtWithEffs['ALPmass']
    listOfEffs_Dip = txtWithEffs['Eff_Dip']
    listOfEffs_Rec = txtWithEffs['Eff_Rec']
    K = -42
    for A in range(len(listOfMass_txt)):
        if math.isclose(listOfMass_txt.tolist()[A], mass_sig, rel_tol=1e-6):
            K = A
    if kindofmass=='Diphoton':
        effForThisMass = listOfEffs_Dip[K]
    elif kindofmass=='Recoil':
        effForThisMass = listOfEffs_Rec[K]
    print('effForThisMass =', effForThisMass, '\n')

    sigmaToUse = d["sigma"]

    w.factory('CBShape:sig('+mass_string_to_extract_dfs+', m0['+str(d["mean"])+'], sigma['+str(sigmaToUse)+'], alpha['+str(d["alpha"])+'], n['+str(d["n"])+'])')
    w.pdf('sig').Print()

    # factory things
    w.factory('lumi['+str(Luminosity/1000)+']')

    w.factory('xsec[0, '+str(xsec_min)+', '+str(xsec_max)+']') #parameter of interest, in pb
    w.factory('sigeff['+str(effForThisMass*(singlePhotonEfficiency**3))+']')
    w.factory('prod:sig_yield(lumi, xsec, sigeff)')

    if bkg_degree==2:
        w.factory('Chebychev:bkg('+mass_string_to_extract_dfs+', {a0[0, -5, 5], a1[0, -5, 5]})')
    elif bkg_degree==3:
        w.factory('Chebychev:bkg('+mass_string_to_extract_dfs+', {a0[0, -5, 5], a1[0, -5, 5], a2[0, -5, 5]})')
    elif bkg_degree==4:
        w.factory('Chebychev:bkg('+mass_string_to_extract_dfs+', {a0[0, -5, 5], a1[0, -5, 5], a2[0, -5, 5], a3[0, -5, 5]})')

    w.factory('bkg_yield['+str(int(nbg))+', 0, 500000]')
    w.factory('SUM::factorymodel(sig_yield*sig, bkg_yield*bkg)')

    pdf = w.pdf('factorymodel')

    # create set of observables (will need it for datasets and ModelConfig later) #??? I don't use it...
    pObs = ROOT.RooRealVar(w.var(mass_string_to_extract_dfs))
    obs = ROOT.RooArgSet("observables")
    obs.add(pObs)

    # create set of global observables (need to be defined as constants!)
    globalObs = ROOT.RooArgSet("global_obs")

    # create set of nuisance parameters
    nuis = ROOT.RooArgSet("nuis")
    nuis.add( w.var("bkg_yield") )
    if bkg_degree==2:
        nuis.add( w.var("a0") )
        nuis.add( w.var("a1") )
    elif bkg_degree==3:
        nuis.add( w.var("a0") )
        nuis.add( w.var("a1") )
        nuis.add( w.var("a2") )
    elif bkg_degree==4:
        nuis.add( w.var("a0") )
        nuis.add( w.var("a1") )
        nuis.add( w.var("a2") )
        nuis.add( w.var("a3") )

    # create set of parameters of interest (POI)
    poi = ROOT.RooArgSet("poi")
    poi.add( w.var("xsec") )

    # fix all other variables in model: everything but observables (glob and non), POI, and nuisance parameters must be constant
    w.var("sigeff").setConstant(True)
    w.var("sigma").setConstant(True)

    # now we need new model configs, with PDF="model"
    b_modelNM = ROOT.RooStats.ModelConfig("B_modelNM", w)
    b_modelNM.SetPdf(w.pdf("factorymodel"))
    b_modelNM.SetObservables(ROOT.RooArgSet(x))
    b_modelNM.SetParametersOfInterest( poi )
    w.var("xsec").setVal(0.0)
    b_modelNM.SetSnapshot( poi )

    # create the alternate (s+b) ModelConfig with given value of s
    s = initialCrossSection
    sb_modelNM = ROOT.RooStats.ModelConfig("S+B_modelNM", w)
    sb_modelNM.SetPdf(w.pdf("factorymodel"))
    sb_modelNM.SetObservables(ROOT.RooArgSet(x))
    sb_modelNM.SetParametersOfInterest( poi )
    w.var("xsec").setVal(s)
    sb_modelNM.SetSnapshot( poi )

    profll = ROOT.RooStats.ProfileLikelihoodTestStat(sb_modelNM.GetPdf())

    # Now compute using asymptotic formula; b is alt, sb is null
    ac = ROOT.RooStats.AsymptoticCalculator(dh_bg, b_modelNM, sb_modelNM)
    ac.SetOneSided(True) # KLUDGE -- should want one sided (true) for limit
    ROOT.RooStats.AsymptoticCalculator.SetPrintLevel(-1)

    ########################################################################

    c = ROOT.TCanvas()
    pl = x.frame(ROOT.RooFit.Title("m(a) = "+str(mass_sig)+" GeV/c^{2}, "+kindofmass+" mass"))
    dh_bg.plotOn(pl)

    nll = w.pdf("factorymodel").createNLL(dh_bg)

    tempMinimizer = ROOT.RooMinimizer(nll)
    tempMinimizer.migrad()
    tempStatus = tempMinimizer.save()
    tempMinimizer.Print()
    Status_List.append(tempStatus.status())

    w.pdf("factorymodel").plotOn(pl, ROOT.RooFit.Components('sig'), ROOT.RooFit.LineColor(ROOT.kRed))
    w.pdf("factorymodel").plotOn(pl, ROOT.RooFit.Components('bkg'), ROOT.RooFit.LineColor(ROOT.kGreen))
    w.pdf("factorymodel").plotOn(pl)
    pl.Draw("same")
    c.Draw()   

    # now let's get the nll values
    nll_obs = nll.getVal()

    w.var('xsec').setVal(0)
    nll_0sig = nll.getVal()

    p0_nosyst = ROOT.TMath.Prob( 2*(nll_0sig-nll_obs), 1)/2
    z_nosyst = ROOT.RooStats.PValueToSignificance(p0_nosyst)
    significanceSelfDefined = np.sqrt( 2 * ( nll_0sig - nll_obs ) )

    NLL_Obs_List.append(nll_obs)
    NLL_0Sig_List.append(nll_0sig)
    PValueMaybe_List.append(p0_nosyst)
    PValueMaybeInSigmas_List.append(z_nosyst)
    SignificanceSelfDefined_List.append(significanceSelfDefined)

    c.SaveAs(SavingPath+'Fits_Pictures/Fit_mass'+str(mass_sig)+
             '_'+kindofmass+
             '_bkgDegree'+str(bkg_degree)+
             '_leftTail'+str(lefttailwindow)+
             '_rightTail'+str(righttailwindow)+
             '.pdf')
    c.Close()

    ################################################################################

    asympResult = ac.GetHypoTest()

    asympResult.SetPValueIsRightTail(False)
    asymp_pb = 1. - asympResult.AlternatePValue() # KLUDGE!!! Needs 1 -   
    asympResult.SetPValueIsRightTail(True)
    asymp_psb = asympResult.NullPValue()
    asymp_pother = asympResult.AlternatePValue()

    asymp_clb = 1. - asymp_pb
    asymp_clsb = asymp_psb
    asymp_cls = 9999.

    if asymp_clb > 0: 
        asymp_cls = asymp_clsb/asymp_clb
        print( "cls = ", asymp_cls )

    # create hypotest inverter passing the desired calculator (hc or ac)
    calc = ROOT.RooStats.HypoTestInverter(ac)
    calc.SetVerbose(False)
    calc.SetConfidenceLevel(0.95)

    useCLs = True
    calc.UseCLs(useCLs)
    if useCLs:
        profll.SetOneSided(True)

    npoints = NPointsToScan  # number of points to scan
    # min and max for scan (better to choose smaller intervals)
    poimin = w.var("xsec").getMin()
    poimax = w.var("xsec").getMax()
    poimin = ParamOfInterestMin
    poimax = ParamOfInterestMax
    print( "Doing a fixed scan  in interval : ", poimin, " , ", poimax)
    calc.SetFixedScan(npoints, poimin, poimax)
    r = calc.GetInterval()

    upperLimit = r.UpperLimit()
    ulError = r.UpperLimitEstimatedError()
    print( "The computed upper limit is: ", upperLimit," +/- ", ulError)

    # compute expected limit
    print("Expected upper limits using b-only model : ")
    print("median limit = " ,      r.GetExpectedUpperLimit(0) )
    print("med. limit (-1 sig) " , r.GetExpectedUpperLimit(-1))
    print("med. limit (+1 sig) " , r.GetExpectedUpperLimit(1))
    print("med. limit (-2 sig) " , r.GetExpectedUpperLimit(-2))
    print("med. limit (+2 sig) " , r.GetExpectedUpperLimit(2))

    UpperLimits_Computed_List.append(upperLimit)

    UpperLimits_List.append(r.GetExpectedUpperLimit(0))
    UpperLimits_Minu1Sigma_List.append(r.GetExpectedUpperLimit(-1))
    UpperLimits_Plus1Sigma_List.append(r.GetExpectedUpperLimit(1))
    UpperLimits_Minu2Sigma_List.append(r.GetExpectedUpperLimit(-2))
    UpperLimits_Plus2Sigma_List.append(r.GetExpectedUpperLimit(2))
    
###############################################################################################################################
###############################################################################################################################

savingNameForTheTXT = SavingPath+'UpperLimitsCLs_ComputedAndMedianAndSigmas_'+kindofmass+'_blockMasses'+'All'+'_bkgDegree'+str(bkg_degree)+'_leftTail'+str(lefttailwindow)+'_rightTail'+str(righttailwindow)+'.txt'
        
txtWithFitInfos = open(savingNameForTheTXT, "w")
txtWithFitInfos.write("ALPmass   ComputedUpperLimit   UpperLimits   UpperLimits_Minu1Sigma   UpperLimits_Plus1Sigma   UpperLimits_Minu2Sigma   UpperLimits_Plus2Sigma   NLL_Obs_List   NLL_0Sig_List   PValueMaybe_List   PValueMaybeInSigmas_List   SignificanceSelfDefined_List   Status_List\n")
for A in range(len(list_of_mass_sig)):
    Pippo = str(list_of_mass_sig[A])+' '+str(UpperLimits_Computed_List[A])+' '+str(UpperLimits_List[A])+' '+str(UpperLimits_Minu1Sigma_List[A])+' '+str(UpperLimits_Plus1Sigma_List[A])+' '+str(UpperLimits_Minu2Sigma_List[A])+' '+str(UpperLimits_Plus2Sigma_List[A])+' '+str(NLL_Obs_List[A])+' '+str(NLL_0Sig_List[A])+' '+str(PValueMaybe_List[A])+' '+str(PValueMaybeInSigmas_List[A])+' '+str(SignificanceSelfDefined_List[A])+' '+str(Status_List[A])+'\n'
    txtWithFitInfos.write(Pippo)
txtWithFitInfos.close()










