#!/usr/bin/env python
# coding: utf-8

###############################################################################################################################

import pandas as pd
import numpy as np
import matplotlib.pyplot as plt

###############################################################################################################################

def printNiceInfos(kindOfMass = 'Diphoton', sizeOfFont=22, xPos=0.97, alignment='right', eeSample=False):
    plt.text(xPos, 0.93, "Belle II simulation", ha=alignment,
            transform=plt.gca().transAxes,
            fontsize=sizeOfFont,
            style='italic',
            alpha=0.95,
            weight='bold',
            )
    plt.text(xPos, 0.85, kindOfMass+" mass", transform=plt.gca().transAxes, fontsize=sizeOfFont, alpha=0.95, ha=alignment)
    plt.text(xPos, 0.77, "$\int\,L\,dt\;=\;"+str(round(Luminosity/1000))[0:3]+"\; pb^{-1}$", transform=plt.gca().transAxes, fontsize=sizeOfFont, alpha=0.95, ha=alignment)
    if eeSample:
        plt.text(xPos, 0.69, "Tracking ineff. = " + str(TrackIneff), ha=alignment,
                transform=plt.gca().transAxes,
                fontsize=sizeOfFont,
                alpha=0.95)
        
def OdinPlotter_Simple(x_label, x_unit, y_label, y_unit, x_input, computed_in, x_min, x_max, y_min, y_max, x_log=False, y_log=False, mainTitle='IfYouSeeThisThereAreProblems', additionalText='', TestShow=0):
    
    baseText = mainTitle
    extraText = '_'+args_kindofmass+additionalText
    
    if x_min==None and x_max==None:
        x_min = int(min(x_input))-1
        if x_min < 0:
            x_min = 0
        x_max = int(max(x_input))+1

    fig, axOmni = plt.subplots()

    axOmni.grid(axis='x', linestyle='--')
    axOmni.grid(axis='y', linestyle='--')

    axOmni.errorbar(x_input, computed_in, fmt='o-', label='CLs computed limit: FAKE DATA', color='black', lw=2, markersize=5)

    axOmni.set_xlabel(x_label+' ['+x_unit+']', horizontalalignment = 'right', x = 1.0)
    if y_unit!='':
        axOmni.set_ylabel(y_label+' ['+y_unit+']', horizontalalignment = 'right', y = 1.0)
    else:
        axOmni.set_ylabel(y_label, horizontalalignment = 'right', y = 1.0)

    if x_log:
        axOmni.set_xscale("log")
        extraText = extraText + '_xLog'
        if x_min!=None:
            if x_min<=0:
                x_min = None
    if y_log:
        axOmni.set_yscale("log")
        extraText = extraText + '_yLog'
        if y_min!=None:
            if y_min<=0:
                y_min = None
    axOmni.tick_params(direction='in', bottom=True, top=True, left=True, right=True, length = 7, width = 2)

    plt.xlim(x_min, x_max)
    plt.ylim(y_min, y_max*1.35)

    printNiceInfos(kindOfMass=args_kindofmass, sizeOfFont=22, xPos=0.97, alignment='right', eeSample=False)
    fig.tight_layout()

    if ShowThePlotsInsteadOfPlottingThem==1 or TestShow==1:
        plt.show()
    else:
        fig.savefig(SavingPath+baseText+extraText+'.pdf')
        plt.close('all')

###############################################################################################################################

ShowThePlotsInsteadOfPlottingThem = 0
SavingPath_Computed = './'
SavingPath = SavingPath_Computed

###############################################################################################################################
###############################################################################################################################

kindOfMassToLookAt = 'Diphoton'

args_kindofmass = kindOfMassToLookAt
args_bkgdegree = 3
args_righttailwindow = 20 # Diphoton
args_lefttailwindow = 30 # 

Luminosity = 496731 #nb-1 =0.51 fb-1

print('args_kindofmass =', args_kindofmass)
numOfInitialEntriesToKill = 0

proxyEnding = '_blockMassesAll_bkgDegree'+str(args_bkgdegree)+'_leftTail'+str(args_righttailwindow)+'_rightTail'+str(args_lefttailwindow)
endingForSaving = proxyEnding

ResultsFromSampleExtraction = pd.read_csv(SavingPath_Computed+'UpperLimitsCLs_ComputedAndMedianAndSigmas_'+args_kindofmass+proxyEnding+'.txt', delim_whitespace=True, header=None, skiprows=[0])
ResultsFromSampleExtraction.columns = ["ALPmass", "ComputedUpperLimit", "UpperLimits_Median", "UpperLimits_Minu1Sigma_Median", "UpperLimits_Plus1Sigma_Median", "UpperLimits_Minu2Sigma_Median", 
                                       "UpperLimits_Plus2Sigma_Median",
                                       "NLL_Obs_Median_List", "NLL_0Sig_Median_List", "PValueMaybe_Median_List", "PValueMaybeInSigmas_Median_List", 
                                       "SignificanceSelfDefined_Median_List", "Status_Median_List"]

Sample_ALPmass = ResultsFromSampleExtraction['ALPmass']
Sample_ComputedUpperLimit = ResultsFromSampleExtraction['ComputedUpperLimit']
Sample_PValue = ResultsFromSampleExtraction['PValueMaybe_Median_List']
Sample_Significance = ResultsFromSampleExtraction['PValueMaybeInSigmas_Median_List']

list_Sample_ALPmass = Sample_ALPmass.tolist()
list_Sample_ComputedUpperLimit = Sample_ComputedUpperLimit.tolist()
list_Sample_PValue = Sample_PValue.tolist()
list_Sample_Significance = Sample_Significance.tolist()

x_label = 'ALP mass'
x_unit = 'GeV/c$^2$'
y_label = '95% CL upper limit on $\sigma(e^+e^- \\longrightarrow a \\gamma)$'
y_unit = 'pb'
xmin_here = 0.15
xmax_here = 1.05

for xLog in [False]:
    for yLog in [False]:
        OdinPlotter_Simple(x_label=x_label, x_unit=x_unit, y_label=y_label, y_unit=y_unit, x_input=list_Sample_ALPmass, 
                           computed_in=list_Sample_ComputedUpperLimit,
                           x_min=xmin_here, x_max=xmax_here, y_min=0, y_max=4, 
                           x_log=xLog, y_log=yLog,
                           mainTitle='UpperLimitsCLs_CS', additionalText=endingForSaving, TestShow=0)

list_Sample_Significance_fixed = []
for x in list_Sample_Significance:
    if x==np.inf:
        x=0
    list_Sample_Significance_fixed.append(x)

x_label = 'ALP mass'
x_unit = 'GeV/c$^2$'
y_label = 'Significance'
y_unit = ''

for xLog in [False]:
    for yLog in [False]:
        OdinPlotter_Simple(x_label=x_label, x_unit=x_unit, y_label=y_label, y_unit=y_unit, x_input=list_Sample_ALPmass, 
                           computed_in=list_Sample_Significance_fixed,
                           x_min=xmin_here, x_max=xmax_here, y_min=0, y_max=4, 
                           x_log=xLog, y_log=yLog,
                           mainTitle='Significance_scan', additionalText=endingForSaving, TestShow=0)

###############################################################################################################################

kindOfMassToLookAt = 'Recoil'

args_kindofmass = kindOfMassToLookAt
args_bkgdegree = 4
args_righttailwindow = 25 # Recoil
args_lefttailwindow = 25 # Recoil

Luminosity = 496731 #nb-1 =0.51 fb-1

print('args_kindofmass =', args_kindofmass)
numOfInitialEntriesToKill = 0

proxyEnding = '_blockMassesAll_bkgDegree'+str(args_bkgdegree)+'_leftTail'+str(args_righttailwindow)+'_rightTail'+str(args_lefttailwindow)
endingForSaving = proxyEnding

ResultsFromSampleExtraction = pd.read_csv(SavingPath_Computed+'UpperLimitsCLs_ComputedAndMedianAndSigmas_'+args_kindofmass+proxyEnding+'.txt', delim_whitespace=True, header=None, skiprows=[0])
ResultsFromSampleExtraction.columns = ["ALPmass", "ComputedUpperLimit", "UpperLimits_Median", "UpperLimits_Minu1Sigma_Median", "UpperLimits_Plus1Sigma_Median", "UpperLimits_Minu2Sigma_Median", 
                                       "UpperLimits_Plus2Sigma_Median",
                                       "NLL_Obs_Median_List", "NLL_0Sig_Median_List", "PValueMaybe_Median_List", "PValueMaybeInSigmas_Median_List", 
                                       "SignificanceSelfDefined_Median_List", "Status_Median_List"]

Sample_ALPmass = ResultsFromSampleExtraction['ALPmass']
Sample_ComputedUpperLimit = ResultsFromSampleExtraction['ComputedUpperLimit']
Sample_PValue = ResultsFromSampleExtraction['PValueMaybe_Median_List']
Sample_Significance = ResultsFromSampleExtraction['PValueMaybeInSigmas_Median_List']

list_Sample_ALPmass = Sample_ALPmass.tolist()
list_Sample_ComputedUpperLimit = Sample_ComputedUpperLimit.tolist()
list_Sample_PValue = Sample_PValue.tolist()
list_Sample_Significance = Sample_Significance.tolist()

x_label = 'ALP mass'
x_unit = 'GeV/c$^2$'
y_label = '95% CL upper limit on $\sigma(e^+e^- \\longrightarrow a \\gamma)$'
y_unit = 'pb'
xmin_here = 8.55
xmax_here = 9.85

for xLog in [False]:
    for yLog in [False]:
        OdinPlotter_Simple(x_label=x_label, x_unit=x_unit, y_label=y_label, y_unit=y_unit, x_input=list_Sample_ALPmass, 
                           computed_in=list_Sample_ComputedUpperLimit,
                           x_min=xmin_here, x_max=xmax_here, y_min=0, y_max=4, 
                           x_log=xLog, y_log=yLog,
                           mainTitle='UpperLimitsCLs_CS', additionalText=endingForSaving, TestShow=0)

list_Sample_Significance_fixed = []
for x in list_Sample_Significance:
    if x==np.inf:
        x=0
    list_Sample_Significance_fixed.append(x)

x_label = 'ALP mass'
x_unit = 'GeV/c$^2$'
y_label = 'Significance'
y_unit = ''

for xLog in [False]:
    for yLog in [False]:
        OdinPlotter_Simple(x_label=x_label, x_unit=x_unit, y_label=y_label, y_unit=y_unit, x_input=list_Sample_ALPmass, 
                           computed_in=list_Sample_Significance_fixed,
                           x_min=xmin_here, x_max=xmax_here, y_min=0, y_max=4, 
                           x_log=xLog, y_log=yLog,
                           mainTitle='Significance_scan', additionalText=endingForSaving, TestShow=0)

###############################################################################################################################























